import os
import tarfile

from dbbackup import utils
from dbbackup.db.base import get_connector
from django.core.files.storage import get_storage_class


def backup_database(database_name):
    connector = get_connector('default')
    filename = connector.generate_filename()
    outputfile = connector.create_dump()
    compressed_file, filename = utils.compress_file(outputfile, filename)
    outputfile = compressed_file
    outputfile.seek(0)
    return outputfile, filename


def backup_media():
    extension = "tar.gz"
    filename = utils.filename_generate(extension, content_type='media')
    # Create tarball
    media_storage = get_storage_class()()
    outputfile = utils.create_spooled_temporary_file()
    tar_file = tarfile.open(name=filename, fileobj=outputfile, mode='w:gz')
    for media_filename in explore_storage(media_storage):
        tarinfo = tarfile.TarInfo(media_filename)
        media_file = media_storage.open(media_filename)
        tarinfo.size = len(media_file)
        tar_file.addfile(tarinfo, media_file)
    # Close the TAR for writing
    tar_file.close()
    # Store backup
    outputfile.seek(0)

    return outputfile, filename


def explore_storage(media_storage):
    """Generator of all files contained in media storage."""
    path = ''
    dirs = [path]
    while dirs:
        path = dirs.pop()
        subdirs, files = media_storage.listdir(path)
        for media_filename in files:
            yield os.path.join(path, media_filename)
        dirs.extend([os.path.join(path, subdir) for subdir in subdirs])


from django.conf.urls import url

try:
  from django.core.urlresolvers import reverse
except ModuleNotFoundError:
  from django.urls import reverse

try:
  from wagtail.wagtailadmin.menu import MenuItem
except ModuleNotFoundError:
  from wagtail.admin.menu import MenuItem

try:
  from wagtail.wagtailcore import hooks
except ModuleNotFoundError:
  from wagtail.core import hooks

from ..views import BackupView


@hooks.register('register_admin_menu_item')
def register_menu_item():
  return MenuItem('Backup DB & Media', reverse('wagtail_backup_view'), classnames='icon icon-download', order=10000)


@hooks.register('register_admin_urls')
def urlconf():
  return [
    url(r'^backup-database-and-media/$', BackupView.as_view(template_name='wagtail/backup_view.html'), name='wagtail_backup_view'),
  ]



from django import forms
from django.conf import settings
from .utils import backup_database, backup_media


class DatabaseBackupForm(forms.Form):
    database = forms.ChoiceField(
        label='Pick a database to backup',
        choices=zip(
            settings.DATABASES.keys(),
            settings.DATABASES.keys()
        )
    )

    def do_backup(self):
        return backup_database(self.cleaned_data['database'])


class MediaBackupForm(forms.Form):

    def do_backup(self):
        return backup_media()



from django.contrib.auth.mixins import LoginRequiredMixin, UserPassesTestMixin
from django.http import HttpResponse
from django.views import View
from django.views.generic.base import TemplateResponseMixin

from .forms import DatabaseBackupForm, MediaBackupForm


class BackupView(LoginRequiredMixin, UserPassesTestMixin, TemplateResponseMixin, View):

    template_name = 'dbbackup_ui/backup_view.html'

    def test_func(self):
        return self.request.user.is_superuser

    def get(self, request, *args, **kwargs):
        context = {
            'database_backup_form': DatabaseBackupForm(),
            'media_backup_form': MediaBackupForm(),
            'title': 'Backup Database and Media'
        }
        return self.render_to_response(self.update_context(context))

    def post(self, request, *args, **kwargs):
        database_backup_form = DatabaseBackupForm(request.POST)
        media_backup_form = MediaBackupForm(request.POST)

        context = {
            'database_backup_form': database_backup_form,
            'media_backup_form': media_backup_form,
        }

        outputfile, filename = None, None
        if 'savebackup' in request.POST and database_backup_form.is_valid():
            outputfile, filename = database_backup_form.do_backup()
        elif 'mediabackup' in request.POST and media_backup_form.is_valid():
            outputfile, filename = media_backup_form.do_backup()

        if outputfile and filename:
            response = HttpResponse(
                outputfile.read(),
                content_type="application/x-gzip"
            )
            response['Content-Disposition'] = 'inline; filename=' + filename
            return response

        return self.render_to_response(self.update_context(context))

    def update_context(self, context):
        context.update({
            'title': 'Backup Database and Media'
        })
        return context


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md') as history_file:
    history = history_file.read()

setup(
    name='django-dbbackup-ui',
    version='0.2.0',
    description="Backup database and media files via Django admin interface (includes Wagtail admin support)",
    long_description=readme + '\n\n' + history,
    author="Tim Kamanin",
    author_email='tim@timonweb.com',
    url='https://github.com/timonweb/django-dbbackup-ui',
    packages=find_packages(),
    package_dir={'dbbackup_ui':
                 'dbbackup_ui'},
    include_package_data=True,
    install_requires=[
        'django-dbbackup>=3.1.3,<3.2'
    ],
    license="BSD license",
    zip_safe=False,
    keywords=[
        'django', 'database', 'media', 'backup',
        'admin', 'wagtail', 'ui', 'download'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Database',
        'Topic :: System :: Archiving',
        'Topic :: System :: Archiving :: Backup',
        'Topic :: System :: Archiving :: Compression'
    ]
)
